import { useCallback, useMemo } from "react";
const createURIStateUpdater = (stateUpdater, parse, serialize, stateIssearch) => (oldURI) => {
    const oldSerialized = stateIssearch ? oldURI.search : oldURI.hash;
    const oldState = parse(oldSerialized);
    const newState = stateUpdater(oldState);
    if (oldState === newState) {
        return oldURI;
    }
    else if (stateIssearch) {
        const search = serialize(newState);
        return Object.assign(Object.assign({}, oldURI), { search });
    }
    else {
        const hash = serialize(newState);
        return Object.assign(Object.assign({}, oldURI), { hash });
    }
};
export const useRoutableState = (props, parse, serialize, stateIsQuery = false) => {
    const { uri: { search, hash }, uriUpdater, } = props;
    const state = useMemo(() => {
        const serialized = stateIsQuery ? search : hash;
        return parse(serialized);
    }, [parse, search, hash]);
    const notify = useCallback((stateUpdater) => {
        uriUpdater(createURIStateUpdater(stateUpdater, parse, serialize, stateIsQuery));
    }, [uriUpdater, parse, serialize, stateIsQuery]);
    return [state, notify];
};
//# sourceMappingURL=useRoutableState.js.map